
# service endpoint authorization parameters
SERVICE_ENDPOINT_AUTHORIZATION_PERSONAL_ACCESS_TOKEN = 'PersonalAccessToken'
SERVICE_ENDPOINT_AUTHORIZATION_SERVICE_PRINCIPAL = 'ServicePrincipal'

# service endpoint types
SERVICE_ENDPOINT_TYPE_GITHUB = 'github'
SERVICE_ENDPOINT_TYPE_AZURE_RM = 'azurerm'
